"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInferenceSnippets = void 0;
var getInferenceSnippets_js_1 = require("./getInferenceSnippets.js");
Object.defineProperty(exports, "getInferenceSnippets", { enumerable: true, get: function () { return getInferenceSnippets_js_1.getInferenceSnippets; } });
