"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isUrl = isUrl;
function isUrl(modelOrUrl) {
    return /^http(s?):/.test(modelOrUrl) || modelOrUrl.startsWith("/");
}
