import type { BaseArgs, InferenceProvider, RequestArgs } from "../../types.js";
/**
 * @deprecated
 */
export interface LegacyAudioInput {
    data: Blob | ArrayBuffer;
    provider?: InferenceProvider;
}
export declare function preparePayload(args: BaseArgs & ({
    inputs: Blob;
} | LegacyAudioInput)): RequestArgs;
//# sourceMappingURL=utils.d.ts.map