export declare const HF_HUB_URL = "https://huggingface.co";
export declare const HF_ROUTER_URL = "https://router.huggingface.co";
export declare const HF_ROUTER_AUTO_ENDPOINT = "https://router.huggingface.co/v1";
export declare const HF_HEADER_X_BILL_TO = "X-HF-Bill-To";
//# sourceMappingURL=config.d.ts.map