"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PublicAIConversationalTask = void 0;
const providerHelper_js_1 = require("./providerHelper.js");
class PublicAIConversationalTask extends providerHelper_js_1.BaseConversationalTask {
    constructor() {
        super("publicai", "https://api.publicai.co");
    }
}
exports.PublicAIConversationalTask = PublicAIConversationalTask;
