import type { ImageTextToVideoInput } from "@huggingface/tasks";
import type { BaseArgs, Options } from "../../types.js";
export type ImageTextToVideoArgs = BaseArgs & ImageTextToVideoInput;
/**
 * This task takes an image and text input and outputs a generated video.
 * Recommended model: Lightricks/LTX-Video
 */
export declare function imageTextToVideo(args: ImageTextToVideoArgs, options?: Options): Promise<Blob>;
//# sourceMappingURL=imageTextToVideo.d.ts.map