/**
 * Special case: provider configuration for a private models provider (OpenAI in this case).
 */
import { BaseConversationalTask } from "./providerHelper.js";
export declare class OpenAIConversationalTask extends BaseConversationalTask {
    constructor();
}
//# sourceMappingURL=openai.d.ts.map