import { BaseConversationalTask } from "./providerHelper.js";

export class PublicAIConversationalTask extends BaseConversationalTask {
	constructor() {
		super("publicai", "https://api.publicai.co");
	}
}
