import { BaseConversationalTask } from "./providerHelper.js";
export declare class PublicAIConversationalTask extends BaseConversationalTask {
    constructor();
}
//# sourceMappingURL=publicai.d.ts.map