/**
 * Return copy of object, only keeping allowlisted properties.
 */
export declare function pick<T, K extends keyof T>(o: T, props: K[] | ReadonlyArray<K>): Pick<T, K>;
//# sourceMappingURL=pick.d.ts.map