import type { TextToVideoInput } from "@huggingface/tasks";
import type { BaseArgs, Options } from "../../types.js";
export type TextToVideoArgs = BaseArgs & TextToVideoInput;
export type TextToVideoOutput = Blob;
export declare function textToVideo(args: TextToVideoArgs, options?: Options): Promise<TextToVideoOutput>;
//# sourceMappingURL=textToVideo.d.ts.map