// Generated file from package.json. Issues importing JSON directly when publishing on commonjs/ESM - see https://github.com/microsoft/TypeScript/issues/51783
export const PACKAGE_VERSION = "4.13.12";
export const PACKAGE_NAME = "@huggingface/inference";
