"use strict";
// https://dev.to/safareli/pick-omit-and-union-types-in-typescript-4nd9
// https://github.com/microsoft/TypeScript/issues/28339#issuecomment-467393437
/**
 * This allows omitting keys from objects inside unions, without merging the individual components of the union.
 */
Object.defineProperty(exports, "__esModule", { value: true });
