"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toArray = toArray;
function toArray(obj) {
    if (Array.isArray(obj)) {
        return obj;
    }
    return [obj];
}
