/**
 * Return copy of object, omitting blocklisted array of props
 */
export declare function omit<T extends object, K extends keyof T>(o: T, props: K[] | K): Pick<T, Exclude<keyof T, K>>;
//# sourceMappingURL=omit.d.ts.map