"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HF_HEADER_X_BILL_TO = exports.HF_ROUTER_AUTO_ENDPOINT = exports.HF_ROUTER_URL = exports.HF_HUB_URL = void 0;
exports.HF_HUB_URL = "https://huggingface.co";
exports.HF_ROUTER_URL = "https://router.huggingface.co";
exports.HF_ROUTER_AUTO_ENDPOINT = `${exports.HF_ROUTER_URL}/v1`;
exports.HF_HEADER_X_BILL_TO = "X-HF-Bill-To";
