import type { BaseArgs, RequestArgs } from "../../types.js";
/**
 * @deprecated
 */
export interface LegacyImageInput {
    data: Blob | ArrayBuffer;
}
export declare function preparePayload(args: BaseArgs & ({
    inputs: Blob;
} | LegacyImageInput)): RequestArgs;
//# sourceMappingURL=utils.d.ts.map