"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setLogger = setLogger;
exports.getLogger = getLogger;
let globalLogger = console;
function setLogger(logger) {
    globalLogger = logger;
}
function getLogger() {
    return globalLogger;
}
