import type { ImageTextToImageInput } from "@huggingface/tasks";
import type { BaseArgs, Options } from "../../types.js";
export type ImageTextToImageArgs = BaseArgs & ImageTextToImageInput;
/**
 * This task takes an image and text input and outputs a new generated image.
 * Recommended model: black-forest-labs/FLUX.2-dev
 */
export declare function imageTextToImage(args: ImageTextToImageArgs, options?: Options): Promise<Blob>;
//# sourceMappingURL=imageTextToImage.d.ts.map