"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isFrontend = void 0;
const isBackend_js_1 = require("./isBackend.js");
exports.isFrontend = !isBackend_js_1.isBackend;
