export const HF_HUB_URL = "https://huggingface.co";
export const HF_ROUTER_URL = "https://router.huggingface.co";
export const HF_ROUTER_AUTO_ENDPOINT = `${HF_ROUTER_URL}/v1`;
export const HF_HEADER_X_BILL_TO = "X-HF-Bill-To";
