import { BaseConversationalTask, BaseTextGenerationTask } from "./providerHelper.js";
export declare class GroqTextGenerationTask extends BaseTextGenerationTask {
    constructor();
    makeRoute(): string;
}
export declare class GroqConversationalTask extends BaseConversationalTask {
    constructor();
    makeRoute(): string;
}
//# sourceMappingURL=groq.d.ts.map